////////////////////////////////////////////////////////////
// BrightnessContrastObject.h
//
// Created by Roman Ryltsov roman@alax.info
// 
// $Id: $

#pragma once

#include <dshow.h>
#include <dmo.h>
#include <dmodshow.h>
#include "DmoBrightnessContrastSample_i.h"

#pragma comment(lib, "strmiids.lib")
#pragma comment(lib, "dmoguids.lib")
#pragma comment(lib, "msdmo.lib")

#if defined(_WIN32_WCE) && !defined(_CE_DCOM) && !defined(_CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA)
#error "Single-threaded COM objects are not properly supported on Windows CE platform, such as the Windows Mobile platforms that do not include full DCOM support. Define _CE_ALLOW_SINGLE_THREADED_OBJECTS_IN_MTA to force ATL to support creating single-thread COM object's and allow use of it's single-threaded COM object implementations. The threading model in your rgs file was set to 'Free' as that is the only threading model supported in non DCOM Windows CE platforms."
#endif

////////////////////////////////////////////////////////////
// Helpers

inline VOID AtlCheck(HRESULT nResult)
{
	if(FAILED(nResult))
		AtlThrow(nResult);
}

inline VOID AtlCheck(BOOL bValue, HRESULT nResult)
{
	if(!bValue)
		AtlThrow(nResult);
}

inline VOID AtlCheck(const VOID* pvValue, HRESULT nResult)
{
	if(!pvValue)
		AtlThrow(nResult);
}

////////////////////////////////////////////////////////////
// CBrightnessContrastObject

class ATL_NO_VTABLE CBrightnessContrastObject :
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CBrightnessContrastObject, &CLSID_BrightnessContrastObject>,
	public IDispatchImpl<IBrightnessContrastObject>,
	public IMediaObject
{
public:

//DECLARE_REGISTRY_RESOURCEID(IDR_BRIGHTNESSCONTRASTOBJECT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

DECLARE_GET_CONTROLLING_UNKNOWN()

BEGIN_COM_MAP(CBrightnessContrastObject)
	COM_INTERFACE_ENTRY(IMediaObject)
	COM_INTERFACE_ENTRY(IBrightnessContrastObject)
	COM_INTERFACE_ENTRY(IDispatch)
END_COM_MAP()

private:

public:
// CBrightnessContrastObject
	static LPCTSTR GetObjectFriendlyName() throw()
	{
		return _T("Alax.Info BrightnessContrast Sample");
	}
	static VOID RegisterMediaObject()
	{
		DMO_PARTIAL_MEDIATYPE pInputMediaTypes[] = 
		{
			{ MEDIATYPE_Video, MEDIASUBTYPE_YUY2 },
		};
		DMO_PARTIAL_MEDIATYPE pOutputMediaTypes[] = 
		{
			{ MEDIATYPE_Video, MEDIASUBTYPE_YUY2 },
		};
		static const DWORD g_nFlags = 0;
		AtlCheck(DMORegister(CStringW(GetObjectFriendlyName()), GetObjectCLSID(), DMOCATEGORY_VIDEO_EFFECT, g_nFlags, _countof(pInputMediaTypes), pInputMediaTypes, _countof(pOutputMediaTypes), pOutputMediaTypes));
	}
	static VOID UnregisterMediaObject()
	{
		AtlCheck(DMOUnregister(GetObjectCLSID(), DMOCATEGORY_VIDEO_EFFECT));
	}
	static HRESULT WINAPI UpdateRegistry(BOOL bRegister) throw()
	{
		_ATLTRY
		{
			if(!bRegister)
				UnregisterMediaObject();
			ATLASSERT(_pAtlModule);
			_pAtlModule->UpdateRegistryFromResource(IDR_BRIGHTNESSCONTRASTOBJECT, bRegister);
			if(bRegister)
				RegisterMediaObject();
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	CBrightnessContrastObject() throw()
	{
		ATLTRACE2(atlTraceRefcount, 4, _T("this 0x%08x\n"), this);
	}
	~CBrightnessContrastObject() throw()
	{
		ATLTRACE2(atlTraceRefcount, 4, _T("this 0x%08x\n"), this);
	}

// CComObjectRootEx
	VOID Lock()
	{
		CComObjectRootEx<CComMultiThreadModel>::Lock();
	}

// IBrightnessContrastObject

// IMediaObject
	STDMETHOD(GetStreamCount)(DWORD* pnInputStreamCount, DWORD* pnOutputStreamCount) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("...\n"));
		_ATLTRY
		{
			AtlCheck(pnInputStreamCount || pnOutputStreamCount, E_POINTER);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetInputStreamInfo)(DWORD nInputStreamIndex, DWORD* pnFlags) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d\n"), nInputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pnFlags, E_POINTER);
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetOutputStreamInfo)(DWORD nOutputStreamIndex, DWORD* pnFlags) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nOutputStreamIndex %d\n"), nOutputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pnFlags, E_POINTER);
			AtlCheck(nOutputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetInputType)(DWORD nInputStreamIndex, DWORD nTypeIndex, DMO_MEDIA_TYPE* pMediaType) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d, nTypeIndex %d\n"), nInputStreamIndex, nTypeIndex);
		_ATLTRY
		{
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			AtlCheck(nTypeIndex >= 0, E_INVALIDARG);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetOutputType)(DWORD nOutputStreamIndex, DWORD nTypeIndex, DMO_MEDIA_TYPE* pMediaType) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nOutputStreamIndex %d, nTypeIndex %d\n"), nOutputStreamIndex, nTypeIndex);
		_ATLTRY
		{
			AtlCheck(nOutputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			AtlCheck(nTypeIndex >= 0, E_INVALIDARG);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(SetInputType)(DWORD nInputStreamIndex, const DMO_MEDIA_TYPE* pMediaType, DWORD nFlags) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d, nFlags 0x%x\n"), nInputStreamIndex, nFlags);
		_ATLTRY
		{
			AtlCheck(pMediaType || (nFlags & DMO_SET_TYPEF_CLEAR), E_INVALIDARG);
			AtlCheck((nFlags & (DMO_SET_TYPEF_TEST_ONLY | DMO_SET_TYPEF_CLEAR)) != (DMO_SET_TYPEF_TEST_ONLY | DMO_SET_TYPEF_CLEAR), E_INVALIDARG);
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(SetOutputType)(DWORD nOutputStreamIndex, const DMO_MEDIA_TYPE* pMediaType, DWORD nFlags) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nOutputStreamIndex %d, nFlags 0x%x\n"), nOutputStreamIndex, nFlags);
		_ATLTRY
		{
			AtlCheck(pMediaType || (nFlags & DMO_SET_TYPEF_CLEAR), E_INVALIDARG);
			AtlCheck((nFlags & (DMO_SET_TYPEF_TEST_ONLY | DMO_SET_TYPEF_CLEAR)) != (DMO_SET_TYPEF_TEST_ONLY | DMO_SET_TYPEF_CLEAR), E_INVALIDARG);
			AtlCheck(nOutputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetInputCurrentType)(DWORD nInputStreamIndex, DMO_MEDIA_TYPE* pMediaType) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d\n"), nInputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pMediaType, E_POINTER);
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetOutputCurrentType)(DWORD nOutputStreamIndex, DMO_MEDIA_TYPE* pMediaType) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nOutputStreamIndex %d\n"), nOutputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pMediaType, E_POINTER);
			AtlCheck(nOutputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetInputSizeInfo)(DWORD nInputStreamIndex, DWORD* pnBufferSize, DWORD* pnMaximalLookAheadBufferSize, DWORD* pnAlignment) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d\n"), nInputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pnBufferSize && pnMaximalLookAheadBufferSize && pnAlignment, E_POINTER);
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetOutputSizeInfo)(DWORD nOutputStreamIndex, DWORD* pnBufferSize, DWORD* pnAlignment) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nOutputStreamIndex %d\n"), nOutputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pnBufferSize && pnAlignment, E_POINTER);
			AtlCheck(nOutputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetInputMaxLatency)(DWORD nInputStreamIndex, REFERENCE_TIME* pnMaximalLatency) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d\n"), nInputStreamIndex);
		//AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
		return E_NOTIMPL;
	}
	STDMETHOD(SetInputMaxLatency)(DWORD nInputStreamIndex, REFERENCE_TIME nMaximalLatency) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d, nMaximalLatency %d\n"), nInputStreamIndex, (ULONG) nMaximalLatency);
		//AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
		return E_NOTIMPL;
	}
	STDMETHOD(Flush)() throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("...\n"));
		_ATLTRY
		{
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(Discontinuity)(DWORD nInputStreamIndex) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d\n"), nInputStreamIndex);
		_ATLTRY
		{
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(AllocateStreamingResources)() throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("...\n"));
		_ATLTRY
		{
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(FreeStreamingResources)() throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("...\n"));
		_ATLTRY
		{
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(GetInputStatus)(DWORD nInputStreamIndex, DWORD* pnFlags) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d\n"), nInputStreamIndex);
		_ATLTRY
		{
			AtlCheck(pnFlags, E_POINTER);
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(Lock)(LONG bLock) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("bLock %d\n"), bLock);
		_ATLTRY
		{
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(ProcessInput)(DWORD nInputStreamIndex, IMediaBuffer* pMediaBuffer, DWORD nFlags, REFERENCE_TIME nTime, REFERENCE_TIME nLength) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nInputStreamIndex %d, nFlags 0x%x\n"), nInputStreamIndex, nFlags);
		_ATLTRY
		{
			AtlCheck(nInputStreamIndex == 0, DMO_E_INVALIDSTREAMINDEX);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}
	STDMETHOD(ProcessOutput)(DWORD nFlags, DWORD nOutputBufferCount, DMO_OUTPUT_DATA_BUFFER* pOutputBuffers, DWORD* pnStatus) throw()
	{
		ATLTRACE2(atlTraceCOM, 4, _T("nFlags 0x%x, nOutputBufferCount %d\n"), nFlags, nOutputBufferCount);
		_ATLTRY
		{
			AtlCheck(nOutputBufferCount == 1, E_INVALIDARG);
			AtlCheck(pOutputBuffers, E_INVALIDARG);
			AtlCheck(pnStatus, E_POINTER);
			// TODO: ...
		}
		_ATLCATCH(Exception)
		{
			return Exception;
		}
		return S_OK;
	}

};

OBJECT_ENTRY_AUTO(__uuidof(BrightnessContrastObject), CBrightnessContrastObject)
